@@ -1,5 +1,5 @@
 from eth2spec.test.context import PHASE1
-from eth2spec.test.helpers.attestations import get_valid_attestation, sign_attestation
+from eth2spec.test.helpers.attestations import get_valid_attestation, sign_attestation, sign_indexed_attestation
 
 
 def get_valid_attester_slashing(spec, state, signed_1=False, signed_2=False):
@@ -17,6 +17,26 @@ def get_valid_attester_slashing(spec, state, signed_1=False, signed_2=False):
     )
 
 
+def get_valid_attester_slashing_by_indices(spec, state, indices_1, indices_2=None, signed_1=False, signed_2=False):
+    if indices_2 is None:
+        indices_2 = indices_1
+
+    assert indices_1 == sorted(indices_1)
+    assert indices_2 == sorted(indices_2)
+
+    attester_slashing = get_valid_attester_slashing(spec, state)
+
+    attester_slashing.attestation_1.attesting_indices = indices_1
+    attester_slashing.attestation_2.attesting_indices = indices_2
+
+    if signed_1:
+        sign_indexed_attestation(spec, state, attester_slashing.attestation_1)
+    if signed_2:
+        sign_indexed_attestation(spec, state, attester_slashing.attestation_2)
+
+    return attester_slashing
+
+
 def get_indexed_attestation_participants(spec, indexed_att):
     """
     Wrapper around index-attestation to return the list of participant indices, regardless of spec phase.
